﻿param($MetricsFolderPath,$ServiceTagFolders)	

$LogLevel = 1
$OMPSInstallKey = "HKLM:\Software\Microsoft\System Center Operations Manager\12\Setup\Powershell\V2"
$regKey = get-item $OMPSInstallKey
$OMPSInstallPath = $regKey.GetValue("InstallDirectory")
$omModule = $OMPSInstallPath + "OperationsManager"
Import-Module -Name $omModule
Import-Module OperationsManager
$scriptname = "DellCMCPerformanceDataInjector"
$logDirectory = "DellSDKCMC_log"
try
{
	$ErrorActionPreference = "SilentlyContinue"
	if ($LogLevel -ne 0) 
	{
		$TempFolder = $env:SystemRoot + "\Temp"
		$LogLocation = $TempFolder + "\" + $logDirectory + "\"
		If(!(Test-Path -path($TempFolder)))
		{
			# create the directory if not present
			New-Item $TempFolder -type directory 
		}

		If(!(Test-Path -path($LogLocation)))
		{
			# create the directory if not present
			New-Item $LogLocation -type directory 
		}
		$Global:LogFileLocation = $LogLocation + $scriptname + ".log"
		If(!(Test-Path -path($LogFileLocation)))
		{
			# create the file if it does not exist
			New-Item $LogFileLocation -type file
		}
		Else
		{
			$logFileSize = Get-ChildItem $LogFileLocation | ForEach-Object {($_.Length/1KB)}
			If ($logFileSize -gt 512)
			{
				# existingLogFile is greater than 512 KB
				$archiveTime = Get-Date -f "yyyy-MM-dd_HH.mm" 
				Rename-Item $LogFileLocation ("ArchivedLog_" + $scriptname + "_" + $archiveTime + ".log")
				New-Item $LogFileLocation -type file
			}                    
		}
	}
}
catch
{
	$ErrorMessage = $_.Exception.Message
	$FailedItem = $_.Exception.ItemName
	$message = "Exception in creating log file. Exception-Message:" + $ErrorMessage + $FailedItem
	#$api.LogScriptEvent("DellServerUnitEventTrigger.ps1",1345,0,$message)
}
Function psDebugLog
{
	param($level, $message)
	if (($level -gt 0) -and ($level -le $LogLevel))
	{
		$currentTime = Get-Date -f "yyyy-MM-dd_HH.mm.ss"
		Out-File -FilePath $LogFileLocation -InputObject ($currentTime + " :: " + $message) -Append -EA SilentlyContinue
	}
}	

function Apply-All-Metrics
{
    param($MetricsFolderPath,$ServiceTagFolders)
    psDebugLog -level 1 -message "CMC/NGM Performance Metrics Data Injector Script is Started"	
    foreach($DeviceID in $ServiceTagFolders)
    {  
		$DeviceName = $DeviceID
		$MetricsFilepath = $MetricsFolderPath + "\" + $DeviceName + "\Metrics.json"
		psDebugLog -level 1 -message "----------------------------------------------------------------------------------------------" 
		psDebugLog -level 1 -message "CMC/NGM Performance Metrics Data Injector Script is processing metrics data for device $DeviceName "
		Apply-Device-Metrics -MetricsFolderPath $MetricsFilepath -Servicetag $DeviceName    
    }

    psDebugLog -level 1 -message ("Performance Metrics script execution completed-----------------------------------------------")
      return "Performance Metrics script execution completed."
}

function InsertMetricsValue ($Object,$InstancePropertiesName,$InstanceDisplayName,$Value,$instance,$device)
{
        if($Value.Contains(" "))
        {
        $v=$Value.Split(" ")
        $Value=$v[0]
        }
        $InstancePerformance = New-Object -Type Microsoft.EnterpriseManagement.Monitoring.CustomMonitoringPerformanceData($Object , $InstancePropertiesName, $Value )
        $InstancePerformance.InstanceName = $Object + " " +  $InstanceDisplayName     #$instance.Key                  
        $device.InsertCustomMonitoringPerformanceData($InstancePerformance)
        psDebugLog -level 1 -message ("Metrics data inserted for Object $Object  Counter " + $InstancePropertiesName  + "  Instance " + $instance.Key + "  metrics value is " + $Value)
}



#Read Metrics from the Metrics data file and Insert Metrics data in SCOM

function Apply-Device-Metrics
{
   
    param($MetricsFolderPath, $Servicetag)
    if ((Get-Item $MetricsFolderPath -ErrorAction Ignore) -eq $null) 
		{
			return -1
		}
	$jsonObj = ConvertFrom-Json -InputObject (Get-Content $MetricsFolderPath -Raw)
    $filtered = $jsonObj |select  TemperatureMetrics, PowerConsumption, System, Fan 
    $DellCMCclassname = 'Dell.SDK.CMC.System'
    $Key  = $Servicetag               
    $DellCMCclassnameInstance =  (Get-SCOMClass -Name  $DellCMCclassname | Get-SCOMClassInstance) 
    $device = $DellCMCclassnameInstance | where {($_.'[Dell.SDK.CMC.System].Key'.Value -eq $Key ) -and ($_.InMaintenanceMode -eq $false )}
    if($device.count -eq 0)
    {
    psDebugLog -level 1 -message ("Not able to insert metrics data from file $DeviceID  as there is no CMC/NGM discovered with Service tag " + $Key)
    continue
    }
    $ComponentHashTable = @{}           
    $filtered.PsObject.Properties | ForEach-Object {$ComponentHashTable.add($_.Name,$_.Value) }
    foreach($Component in $ComponentHashTable.Keys)
     {  
		if($Component -eq "TemperatureMetrics")
            {  
              $instance = $ComponentHashTable.Item($Component)
              if($instance.Key.contains("Temperature"))
                 {

                 $value = $instance.instantaneousTemperature
				  $InstancePropertiesName = "Chassis Inlet Temperature (Celsius)"
				  $InstanceDisplayName ="Chassis Inlet Temperature"
				  InsertMetricsValue -Object $Key -InstancePropertiesName $InstancePropertiesName -InstanceDisplayName $InstanceDisplayName -value $value -instance $instance -device $device
				 

                  $value = $instance.peakTemperature
                 $InstancePropertiesName = "Peak Temperature (Celsius)"
                 $InstanceDisplayName= "PeakTemperature"
                  InsertMetricsValue -Object $Key -InstancePropertiesName $InstancePropertiesName -InstanceDisplayName $InstanceDisplayName -value $value -instance $instance -device $device
                 }  
            }

		if($Component -eq "Fan")
            {  
              foreach($instance in $ComponentHashTable.Item($Component))
              {
                  if($instance.Key.contains("Fan"))
					 {
					 $value = $instance.RPM
					 $InstancePropertiesName = "Fan speed (RPM)"
					 $InstanceDisplayName= $instance.Key
					  InsertMetricsValue -Object $Key -InstancePropertiesName $InstancePropertiesName -InstanceDisplayName $InstanceDisplayName -value $value -instance $instance -device $device
					 }
               }    
        
            }




        if($Component -eq "PowerConsumption")
			{  
			   $instance = $ComponentHashTable.Item($Component)
			   if($instance.Key.contains("Power"))
				 {
				  $value = $instance.peakPower
				  $InstancePropertiesName = "Peak Power(Watts)"
				  $InstanceDisplayName="PeakPower"
				  InsertMetricsValue -Object $Key -InstancePropertiesName $InstancePropertiesName -InstanceDisplayName $InstanceDisplayName -value $value -instance $instance -device $device
				  $value = $instance.systemEnergyConsumption
				  $InstancePropertiesName = "System Energy Consumption(Watts)"
				  $InstanceDisplayName="System Energy Consumption"
				  InsertMetricsValue -Object $Key -InstancePropertiesName $InstancePropertiesName  -InstanceDisplayName $InstanceDisplayName -value $value -instance $instance -device $device

				 } 
			}   
			
		if($Component -eq "System")
			{  
				  foreach($instance in $ComponentHashTable.Item($Component))
				  {
                     if($instance.PwrInputSystemConsumption -ne $null)
                     {
					 $value = $instance.PwrInputSystemConsumption
					 $InstancePropertiesName = "Chassis Total Power Consumption (Watts)"
					 $InstanceDisplayName= ""
					  InsertMetricsValue -Object $Key -InstancePropertiesName $InstancePropertiesName -InstanceDisplayName $InstanceDisplayName -value $value -instance $instance -device $device
					 }
				   }    
	
			}                
    }
  } 
   

   
  
Apply-All-Metrics -MetricsFolderPath $MetricsFolderPath -ServiceTagFolders $ServiceTagFolders


  
